
/*
 * ATMMultiVC.c: Console demo application.
 *
 *  Send and Receive frames over a certain number of VCs
 *  The VCs can be periodicaly closed and opened
 *  Works with both WDM and NDIS5 Proatm drivers
 */
 
#include <winsock2.h>
#include <windows.h>
//#include <ansi_c.h> // For LabWindows environment
#include <stdio.h>
#include "proatmapi.h"



#define MAX_USER_VCS 16
#define MAX_USER_BUFFER_SIZE 4096

typedef struct _USER_VCS
{
	int vpi;
	int vci;
	int vcHandle;
}USER_VCS;

USER_VCS userVCS[MAX_USER_VCS];

char writeBuffer[MAX_USER_BUFFER_SIZE];
char readBuffer[MAX_USER_BUFFER_SIZE];

int nVPI=0;
int nVCI=0;
int nCount =0;

HANDLE hBreakEvent, hCloseReadEvent, hCloseWriteEvent;
BOOL	bBreakExecuted=FALSE;
ULONG ReceivedFrameCount[MAX_USER_VCS];
ULONG SentFrameCount[MAX_USER_VCS];

BOOL GetArguments(int argc, char* argv[])
{
	if (argc < 4)
		return FALSE;

	nVPI = atoi(argv[1]);
	nVCI = atoi(argv[2]);
	nCount = atoi(argv[3]);
	return TRUE;
}

void CloseAllVCs()
{
	int i;
	for (i=0; i< nCount; i++)
	{
		if (userVCS[i].vcHandle >=0)
		{
			proatmCloseVC(userVCS[i].vcHandle);
			printf("VC(%d.%d) closed\n",userVCS[i].vpi, userVCS[i].vci );
		}
	}
}

static BOOL WINAPI BreakHandlerRoutine(DWORD dwCtrlType)
{
	BOOL   bReturnValue = FALSE;

	if(!bBreakExecuted)
	{
		switch (dwCtrlType)
		{
		case (CTRL_C_EVENT):
		case (CTRL_CLOSE_EVENT):
		case (CTRL_BREAK_EVENT):
			CloseAllVCs();
			SetEvent(hBreakEvent);		
			bReturnValue = TRUE;
			break;
		}
	}
	
	return (bReturnValue);

}

// 
// Send PDUs to VCs randomly selected
//

#define WRITE_LENGTH 512

static DWORD WriteThread(DWORD param)
{	
	int i;
	
	// Use a small delay to fairly share time between send and receive threads
	while (WaitForSingleObject(hCloseWriteEvent,0) != WAIT_OBJECT_0) 
	{
		int vcToWrite = rand() % nCount;
		{
			// Send 50 PDUs to the randomly selected VC
			for (i=0; i < 10;i ++)
			{
				if (userVCS[vcToWrite].vcHandle >=0)
				{					
					if ((proatmWritePDU(userVCS[vcToWrite].vcHandle, writeBuffer, WRITE_LENGTH)) == WRITE_LENGTH)
						SentFrameCount[vcToWrite]++;
//						printf("%d bytes sent on VC(%d.%d)\n", bytesWritten, userVCS[vcToWrite].vpi, userVCS[vcToWrite].vci );
					else
					{
//						printf("Could not send %d bytes on VC(%d.%d)\n", MAX_USER_BUFFER_SIZE, userVCS[vcToWrite].vpi, userVCS[vcToWrite].vci );
						break;
					}
				}
			}
			Sleep(1);
		}
	}
	return 0;
}
static DWORD ReadThread(DWORD param)
{
	int i;
	
	// Use a small delay to fairly share the time between send and receive threads
	while (WaitForSingleObject(hCloseReadEvent,1) != WAIT_OBJECT_0)
	{
		for (i=0; i< nCount; i++)
		{
			int bytesRead;
			while((bytesRead=proatmReadPDU(userVCS[i].vcHandle, readBuffer, MAX_USER_BUFFER_SIZE)) > 0)
			{
				ReceivedFrameCount[i]++;
//				printf("%d bytes received on VC(%d.%d)\n", bytesRead, userVCS[i].vpi, userVCS[i].vci );
			}
		}
	}
	return 0;
}

void usage (void)
{
	printf ("\natmmultivc <VPI> <VCI> <Number of VCs> \n");
	Sleep(3000);
}

int main(int argc, char* argv[])
{	
	int i,j;
	HANDLE hReadThread, hWriteThread;
	ULONG OldReceivedFrameCount[MAX_USER_VCS];
	ULONG OldSentFrameCount[MAX_USER_VCS];
	int CloseCount = 0;
	
	// Get VPI VCI arguments
	if (!GetArguments(argc, argv)) 
	{
		usage ();
		return -1;
	}

	// Check the number of VCs
	if (nCount > MAX_USER_VCS)
		nCount =MAX_USER_VCS;

	// Look for all PROATM adapters
	if(proatmInitAdapters() <=0) 
	{
		usage ();
		return -2;
	}

	// Try opening all specified VCs on the first adapter
	j=0;
	for (i=0; i< nCount; i++)
	{
		userVCS[j].vpi = nVPI;
		userVCS[j].vci = nVCI+i; 
		
		ReceivedFrameCount[i]=0;
		SentFrameCount[i]=0;
		
		OldReceivedFrameCount[i]=0;
		OldSentFrameCount[i]=0;
		
		userVCS[j].vcHandle= proatmOpenVC(0,userVCS[i].vpi, userVCS[i].vci);
		if (userVCS[i].vcHandle >=0)
		{	
			printf("VC(%d.%d) is open\n",userVCS[j].vpi, userVCS[j].vci );
			j++;
		}
		else
		{
			// Error
			printf("Could not open VC(%d.%d)\n", userVCS[j].vpi, userVCS[j].vci);
		}
		Sleep(100);
	}
	nCount = j;
	
	if (nCount == 0)
		return -3; // Could not open any VC
	
	SetConsoleCtrlHandler(BreakHandlerRoutine, TRUE );

	hBreakEvent      = CreateEvent(NULL, TRUE, FALSE, NULL);
	hCloseReadEvent  = CreateEvent(NULL, TRUE, FALSE, NULL);
	hCloseWriteEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	hWriteThread=CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)WriteThread, NULL, 0, NULL);
	hReadThread=CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)ReadThread, NULL, 0, NULL);

	while ( WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
	{
#define CLOSE_VC 1

#if CLOSE_VC
		Sleep(2000);		
		// Close a ramdomly choosen vc and open it again
		// int vcToClose = rand() % nCount;
		// proatmCloseVC(userVCS[vcToClose].vcHandle);
		// printf("VC(%d.%d): closed\n",userVCS[vcToClose].vpi, userVCS[vcToClose].vci );
		
		// Close all VCs
		for (i=0; i< nCount; i++)
		{
			proatmCloseVC(userVCS[i].vcHandle);
			Sleep(100);
		}
		printf ("All VCs are closed\n");
#else
		Sleep(1000);
#endif

		for (i=0; i< nCount; i++)
		{
			printf("VC(%d.%d): Frames received: %d - Frames sent: %d\n",
				userVCS[i].vpi, userVCS[i].vci,
				ReceivedFrameCount[i]- OldReceivedFrameCount[i], 
				SentFrameCount[i]- OldSentFrameCount[i] );

			OldReceivedFrameCount[i]= ReceivedFrameCount[i];
			OldSentFrameCount[i]= SentFrameCount[i];
		}

#if CLOSE_VC
		Sleep(200);
		// userVCS[vcToClose].vcHandle = proatmOpenVC(0,userVCS[vcToClose].vpi, userVCS[vcToClose].vci);
		// if (userVCS[vcToClose].vcHandle >=0)
		//	 printf("VC(%d.%d): opened\n",userVCS[vcToClose].vpi, userVCS[vcToClose].vci );
		// else
		//	 printf("VC(%d.%d): open failed\n",userVCS[vcToClose].vpi, userVCS[vcToClose].vci);

		// Open all VCs
		for (i=0; i< nCount; i++)
		{	
			Sleep(100);	
			userVCS[j].vcHandle= proatmOpenVC(0,userVCS[i].vpi, userVCS[i].vci);
			if (userVCS[i].vcHandle < 0)
				printf("Could not open VC(%d.%d)\n", userVCS[j].vpi, userVCS[j].vci);
		}
		printf ("All VCs are open\n");
#endif
	}

	// Close send and receive threads
	SetEvent(hCloseReadEvent);
	SetEvent(hCloseWriteEvent);

	// Wait for thread exit. 
	WaitForSingleObject(hReadThread, INFINITE);	
	WaitForSingleObject(hWriteThread, INFINITE);

	proatmExitAdapters();

	return 0;
}

